#!/usr/bin/python
#Import necessary modules
import os
import sys,time,re
import webbrowser
from PyQt4 import QtGui,QtCore
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from pyRadio_ui import *
from pyRadio_ui_portrait import *
from radioplayer import RadioPlayer

import dbus
from dbus.mainloop.glib import DBusGMainLoop

__title__ = "Pandora"
#class for the "sleep" function -- needs a way to cancel properly
class snooze(QtCore.QThread):
	__pyqtSignals__ = ( "shutdown" )
	def __init__(self,ktime,cont=None):
		self.ktime = ktime
		self.cont = cont
		QtCore.QThread.__init__(self)
	def run(self):
		time.sleep(self.ktime*60)
		if self.cont:
			self.emit(QtCore.SIGNAL("shutdown"))

#class for reading configuration files
class Config:
	def __init__(self):
		self.user = ""
		self.password = ""
		self.color = ""
		self.lastplayed = ""
		self.format = "mp3"
		#I loop through possible configuration options in this array
		self.config_opts = ['user','password','format']

	def write(self): #write the configuration files
		if not os.path.exists(os.path.expanduser("~")+'/.config'):
			os.mkdir(os.path.expanduser("~")+'/.config')

		if not os.path.exists(os.path.expanduser("~")+'/.config/pyradio'):
			os.mkdir(os.path.expanduser("~")+'/.config/pyradio')
		newfile = "user="+self.user+"\npassword="+self.password+"\n"+"format="+self.format+"\n"
		f = open(os.path.expanduser("~")+'/.config/pyradio/config', 'w')
		f.write(newfile)
		f.close()

	def load(self): #load the configuration files
		if os.path.exists('/etc/hildon/theme/colors.config'): #get system colors
			f = open('/etc/hildon/theme/colors.config')
			for line in f:
				if re.match("ActiveTextColor\s*=",line):
					self.color = re.match("ActiveTextColor\s*=(.+)",line).group(1).lstrip().rstrip()
				
			if self.color == "":
				self.color = "rgb(255, 255, 255)"

		else: #default to white
			self.color = "rgb(255, 255, 255)"
		
		#open main configuration file
		if os.path.exists(os.path.expanduser("~")+'/.config/pyradio/config'):
			f = open(os.path.expanduser("~")+'/.config/pyradio/config', 'r')
			for line in f:
				line = line.replace(" ", "")
				
				if re.match("^#.*",line) is not None: #skip comments
					continue
				
				for opt in self.config_opts: #check each line for an option
					if re.match("^"+opt+"=(.+)$",line) is not None:
						setattr(self,opt,self.get_var(opt,line))
			f.close()

		#get lastplayed station to auto-start
		if os.path.exists(os.path.expanduser("~")+'/.config/pyradio/lastplayed'):
			f = open(os.path.expanduser("~")+'/.config/pyradio/lastplayed', 'r')
			self.lastplayed = f.readline().replace("\n","")
			f.close()

	def get_var(self,var,line):
		r = re.match("^"+var+"=(.+)$",line)
		return r.group(1)
		
class Panda(QtGui.QMainWindow): #main class
	def __init__( self, parent=None ):
		QtGui.QWidget.__init__(self, parent)
		self.ui = None 
		self.authenticated = False
		self.c = Config() 
		self.c.load()
		self.player = RadioPlayer()
		self.player.format=self.c.format
		self.player.start()
		self.ui = Ui_pyRadio()
		self.ui.setupUi(self)
		self.portrait = False
		self.os = self.player.get_product()
		if self.os != "RX-51": #check for N900 
			self.ui.albumArt.setStyleSheet("background-color: rgb(255, 255, 255);")

		self.ui.btnHate.setStyleSheet("background-color: rgb(100, 100, 100);\nborder: rgb(255, 255, 255);\nbackground-image: url(:/img/img/thumbdwn_small.png);")

		if not self.c.lastplayed == "": #auto-play last station
			self.player.playStation = self.c.lastplayed
		try : #test for rotation and set auto-rotation on
			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
			self.connect(QApplication.desktop(), SIGNAL("resized(int)"), self.orientationChanged );
			screenGeometry=QApplication.desktop().screenGeometry();
			if screenGeometry.width() > screenGeometry.height():
				self.portrait = False
			else:
				self.portrait = True
			print" Auto rotation on"
		except:print "could not autorotate on"
		self.skip = 0
		self.s = snooze(0)
		self.createStations = []
		self.count = 0
		self.connect_signals() #called every time orientation changes to connect slots
		self.auth()
		try : #test for FM transmitter.
			sysbus = dbus.SystemBus()
			self.fmtx = sysbus.get_object('com.nokia.FMTx', '/com/nokia/fmtx/default', False)
			self.fmtx_iface = dbus.Interface(self.fmtx, dbus_interface='org.freedesktop.DBus.Properties')

			state = self.fmtx_iface.Get("com.nokia.FMTx.Device", "state")
			if state == 'enabled':
				self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
			else:
				self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
		except: print "No FM Transmitter."

	def orientationChanged ( self ): #called when N900 is rotated
		screenGeometry=QApplication.desktop().screenGeometry();
		if screenGeometry.width() > screenGeometry.height(): #landscape
			self.ui = Ui_pyRadio()
			self.ui.setupUi(self)
			self.connect_signals()
			self.portrait = False
			if not self.player.curSong == 0:
				self.setUi ( self.player.curSong )
				self.buildPlaylist()
		else: #portrait
			self.ui = Ui_pyRadio_portrait()
			self.ui.setupUi(self)
			self.connect_signals()
			self.portrait = True
			if not self.player.curSong == 0:
				self.setUi ( self.player.curSong )
				self.buildPlaylist()
		print "Orientation Changed"
		self.ui.lblTime.setWordWrap( True )

	def connect_signals( self ): #connect all signals
		if not self.authenticated:
			self.ui.btnPlay.setEnabled( False )
			self.ui.listStations.setEnabled( False )
			self.ui.btnLove.setEnabled( False )
			self.ui.btnHate.setEnabled( False )
			self.ui.btnSleep.setEnabled( False )
			self.ui.btnNext.setEnabled( False )
		QtCore.QObject.connect(self.ui.listStations, QtCore.SIGNAL('activated(int)'), self.playStation)
		QtCore.QObject.connect(self.ui.btnNext, QtCore.SIGNAL('pressed()'), self.nextDown)
		QtCore.QObject.connect(self.ui.btnNext, QtCore.SIGNAL('released()'), self.nextUp)
		QtCore.QObject.connect(self.ui.btnHate, QtCore.SIGNAL('pressed()'), self.hateDown)
		QtCore.QObject.connect(self.ui.btnHate, QtCore.SIGNAL('released()'), self.hateUp)
		QtCore.QObject.connect(self.ui.btnNext, QtCore.SIGNAL('clicked()'), self.playNextSong)
		QtCore.QObject.connect(self.ui.btnHate, QtCore.SIGNAL('clicked()'), self.doHate)
		QtCore.QObject.connect(self.ui.btnPlay, QtCore.SIGNAL('clicked()'), self.doPlayPause)
		QtCore.QObject.connect(self.ui.btnLove, QtCore.SIGNAL('clicked()'), self.doLove)
		QtCore.QObject.connect(self.ui.btnSleep, QtCore.SIGNAL('clicked()'), self.tiredSong)
		QtCore.QObject.connect(self.player,QtCore.SIGNAL("PlayNext"), self.playNextSong)
		QtCore.QObject.connect(self.player,QtCore.SIGNAL("LoveHate"), self.sigLoveHate)
		QtCore.QObject.connect(self.player,QtCore.SIGNAL("stations"), self.getStations)
		QtCore.QObject.connect(self.player,QtCore.SIGNAL("deleted"), self.stationDeleted)
		QtCore.QObject.connect(self.player,QtCore.SIGNAL("tired"), self.sigTired)
		QtCore.QObject.connect(self.player,QtCore.SIGNAL("authSuccess"), self.sigAuth)
		QtCore.QObject.connect(self.player,QtCore.SIGNAL("authFail"), self.sigAuth)
		QtCore.QObject.connect(self.player,QtCore.SIGNAL("stationCreated"), self.stationCreated)
		QtCore.QObject.connect(self.player,QtCore.SIGNAL("duration"), self.setDuration)
		QtCore.QObject.connect(self.player,QtCore.SIGNAL("state(QString)"), self.doPlayPause)
		QtCore.QObject.connect(self.s, QtCore.SIGNAL('shutdown'), self.closeEvent)
		QtCore.QObject.connect(self.ui.actionDelete, QtCore.SIGNAL('triggered()'), self.deleteStation)
		QtCore.QObject.connect(self.ui.actionEnable_FM_Radio, QtCore.SIGNAL('triggered()'), self.doFM)
		QtCore.QObject.connect(self.ui.actionSnooze, QtCore.SIGNAL('triggered()'), self.setSnooze)
		QtCore.QObject.connect(self.ui.actionCreate, QtCore.SIGNAL('triggered()'), self.createStation)
		QtCore.QObject.connect(self.ui.actionDonate, QtCore.SIGNAL('triggered()'), self.open_donate)
		QtCore.QObject.connect(self.ui.actionAdd_Seed, QtCore.SIGNAL('triggered()'), self.doSeed)
		QtCore.QObject.connect(self.ui.actionSettings, QtCore.SIGNAL('triggered()'), self.openSettings)

	def openSettings( self ): #Build and open settings dialog box.
		self.d = QDialog(self)
		self.d.setParent(self)
		self.settings = {} #Store settings in a hash, it's easier to read in my opinion.
		if self.os == "RX-51":
			if not self.portrait:
				self.d.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
			else:
				self.d.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
		vbox = QVBoxLayout()
		hLayout = QHBoxLayout()
		hLayout2 = QHBoxLayout()
		hLayout3 = QHBoxLayout()
		hLayout4 = QHBoxLayout()
		hLayout5 = QHBoxLayout()
		self.settings['lblUsername'] = QLabel("Username:")
		self.settings['lineUser'] = QLineEdit()
		self.settings['lblUsername'].setBuddy(self.settings['lineUser'])
		self.settings['lineUser'].setText(self.c.user)
		self.settings['lblPass'] = QLabel("Password:")
		self.settings['linePass'] = QLineEdit()
		self.settings['lblPass'].setBuddy(self.settings['linePass'])
		self.settings['linePass'].setText(self.c.password)
		self.settings['linePass'].setEchoMode(QLineEdit.Password)
		self.settings['btnGroup'] = QButtonGroup()
		self.settings['radioHigh'] = QRadioButton("High (Pandora One)")
		self.settings['radioMed'] = QRadioButton("Medium (MP3)")
		self.settings['radioLow'] = QRadioButton("Low (AAC)")
		self.settings['btnGroup'].addButton(self.settings['radioHigh'])
		self.settings['btnGroup'].addButton(self.settings['radioMed'])
		self.settings['btnGroup'].addButton(self.settings['radioLow'])
		if self.c.format == "mp3-hifi":
			self.settings['radioHigh'].setChecked(1)
		elif self.c.format == "mp3":
			self.settings['radioMed'].setChecked(1)
		elif self.c.format == "aacplus":
			self.settings['radioLow'].setChecked(1)
		else:
			self.settings['radioMed'].setChecked(1)
		self.settings['btnGroup2'] = QButtonGroup()
		self.settings['radioAuto'] = QRadioButton("Automatic")
		self.settings['radioPort'] = QRadioButton("Portrait")
		self.settings['radioLand'] = QRadioButton("Landscape")
		if not self.portrait:
			self.settings['radioAuto'].setChecked(1)
		else:
			self.settings['radioPort'].setChecked(1)
		self.settings['btnGroup2'].addButton(self.settings['radioAuto'])
		self.settings['btnGroup2'].addButton(self.settings['radioPort'])
		self.settings['btnGroup2'].addButton(self.settings['radioLand'])
		a = QPushButton("Accept")
		b = QPushButton("Cancel")
		hLayout.addWidget(self.settings['lblUsername'])
		hLayout.addWidget(self.settings['lineUser'])
		hLayout2.addWidget(self.settings['lblPass'])
		hLayout2.addWidget(self.settings['linePass'])
		hLayout3.addWidget(self.settings['radioHigh'])
		hLayout3.addWidget(self.settings['radioMed'])
		hLayout3.addWidget(self.settings['radioLow'])
		hLayout4.addWidget(self.settings['radioAuto'])
		hLayout4.addWidget(self.settings['radioPort'])
		hLayout4.addWidget(self.settings['radioLand'])
		hLayout5.addWidget(a)
		hLayout5.addWidget(b)
		a.clicked.connect(self.setSettings)
		b.clicked.connect(self.d.close)
		vbox.addLayout(hLayout)
		vbox.addLayout(hLayout2)
		vbox.addLayout(hLayout3)
		if self.player.os == "RX-51":
			vbox.addLayout(hLayout4)
		vbox.addLayout(hLayout5)
		self.d.setLayout(vbox)
		self.d.setWindowTitle("pyRadio Settings:")
		self.d.show()

	def setSettings ( self ): #Set settings.
		changed = False
		if self.c.user != self.settings['lineUser'].text() or self.c.password != self.settings['linePass'].text():
			self.c.user = self.settings['lineUser'].text()
			self.c.password = self.settings['linePass'].text()
			self.auth()
			changed = True
		if self.settings['radioHigh'].isChecked():
			format = 'mp3-hifi'
		elif self.settings['radioMed'].isChecked():
			format = 'mp3'
		elif self.settings['radioLow'].isChecked():
			format = 'aacplus'
		if format != self.c.format:
			self.c.format=format
			self.player.format=format
			self.player.playlist = []
			self.player.getPlaylist = 1 #Get a new playlist with the new format!
			changed = True
		if self.os == "RX-51":
			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
		try:
			if self.settings['radioAuto'].isChecked():
				self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
			elif self.settings['radioPort'].isChecked():
				self.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
			elif self.settings['radioLand'].isChecked():
				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
		except: print "No Rotation."
		self.settings = None
		if changed:
			self.c.write()
		self.d.close()

	def doSeed( self ):
		if not self.player.playing:
			QtGui.QMessageBox.warning(self, "Not Playing", "You must be listening to a song to seed it.")
			return

		self.d = QDialog(self)
		self.d.setParent(self)
		if self.os == "RX-51":
			if not self.portrait:
				self.d.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
			else:
				self.d.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
		vbox = QVBoxLayout()
		hLayout = QHBoxLayout()
		m = QPushButton("Seed Artist")
		p = QPushButton("Seed Song")
		c = QPushButton("Close")
		hLayout.addWidget(m)
		hLayout.addWidget(p)
		hLayout.addWidget(c)
		m.clicked.connect(lambda: self.addSeed("artist"))
		p.clicked.connect(lambda: self.addSeed("song"))
		c.clicked.connect(self.d.close)
		vbox.addLayout(hLayout)
		self.d.setLayout(vbox)
		self.d.setWindowTitle("Add Station Seed:")
		self.d.show()

	def addSeed( self, seed=None ):
		if self.player.playing:
			self.player.seedSong = seed
		self.d.close()
		if self.os == "RX-51":
			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
 
	def open_donate( self ): #Donations are AWESOME :)
		self.d = QDialog(self)
		self.d.setParent(self)
		if self.os == "RX-51":
			if not self.portrait:
				self.d.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
			else:
				self.d.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
		vbox = QVBoxLayout()
		hLayout = QHBoxLayout()
		m = QPushButton("pyRadio")
		p = QPushButton("Get Pandora One") #Let's support our radio host :)
		c = QPushButton("Close")
		hLayout.addWidget(m)
		hLayout.addWidget(p)
		hLayout.addWidget(c)
		action = lambda: self.donate('radio')
		action1 = lambda: self.donate('pandora')
		m.clicked.connect(action)
		p.clicked.connect(action1)
		c.clicked.connect(self.d.close)
		vbox.addLayout(hLayout)
		self.d.setLayout(vbox)
		self.d.setWindowTitle("Donate to:")
		self.d.show()

	def donate( self, who=None ):
		if who == 'radio':		
			webbrowser.open_new('https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=BWX2M54JBJ2T2')
		elif who == 'pandora':
			webbrowser.open_new('http://www.pandora.com/pandora_one')
		self.d.close()

	def stationCreated( self ): #Station Created Successfully
		ok = QtGui.QMessageBox.warning(self,'Station Created','Station Created, your station list will reload.')
		self.ui.listStations.clear()
		self.player.getStatlist = 1
	
	def setDuration( self ): #Set the song position/duration
		if (self.player.duration-self.player.position) < self.player.duration:
			position = "%02d:%02d" % divmod(self.player.duration - self.player.position, 60) + " / " + "%02d:%02d" % divmod(self.player.duration, 60)
			self.ui.lblTime.setText(position + "\nAlbum: " + self.player.curSong[1]["album"])

	def createStation( self ): #Start the create station sequence
		if self.os == "RX-51":
			if not self.portrait:
				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
			else:
				self.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
		station, ok = QtGui.QInputDialog.getText(self,"Artist or Track Name", "Enter an Artist or Track Name:")
		if len(station)>0:
			self.ui.albumArt.setHtml("Please wait..... ")
			ret = self.player.search(station)
			if len(ret["artists"])>0 or len(ret["songs"])>0:
				if len(ret["artists"])>0 and len(ret["songs"])>0:
					what = QtGui.QMessageBox.question(self, 'Artist or Track?', 'Is this an artist or track name?','Artist','Track')
					if what == 0:
						songs = ret["artists"]
						title = "artistName"
					else:
						songs = ret["songs"]
						title = "songTitle"
				else:
					if len(ret["artists"])>0:
						songs = ret["artists"]
						title = "artistName"
					else:
						songs = ret["songs"]
						title = "songTitle"

				self.ui.listStations.clear() 
				#For lack of room, populate station list with new stations.  I want to change this to be it's own QDialog in the future.
				for s in songs:
					self.ui.listStations.addItem(s[title])
					self.createStations = songs
			ok = QtGui.QMessageBox.warning(self,'Select Match','Select the closest title from the station drop down.')
		if self.os == "RX-51":
			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)

	def stationDeleted( self ): #Delete station
		ok = QtGui.QMessageBox.warning(self, "Station Deleted", "Station Deleted.")

	def sigTired( self ): #Song marked tired successfully 
		ok = QtGui.QMessageBox.warning(self, "Tired", "Song marked tired.")
		self.playNextSong()

	def deleteStation(self): #Start the delete station process
		if self.os == "RX-51":
			if not self.portrait:
				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
			else:
				self.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
		reply = QtGui.QMessageBox.question(self, 'Delete Station',
			"Really delete "+self.ui.listStations.currentText()+"?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
		if reply == QtGui.QMessageBox.Yes:
			self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
			self.player.stopPlayBack()
			self.player.deleteStation = 1
			self.ui.albumArt.setHtml("Please wait...")
		if self.os == "RX-51":
			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)

	def tiredSong(self): #Start tired song process and wait for signal from radio backend
		check = QtGui.QMessageBox.question(self, 'Are you sure?', 'Are you tired of the song ' + self.player.curSong[1]["title"] + '?','No','Yes')
		if check == 1:
			self.player.tiredSong = 1

	def setSnooze(self): #Set tired
		if self.os == "RX-51":
			if not self.portrait:
				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
			else:
				self.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
		ktime,ok = QtGui.QInputDialog.getText(self,"Snooze", "Enter a time, in minutes, to shutdown:")
		if ok:
			self.s.cont = 1
			self.s.ktime = int(ktime)
			self.s.start()
		if self.os == "RX-51":
			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)

	def doFM(self): #Turn on FM transmitter
		state = self.fmtx_iface.Get("com.nokia.FMTx.Device", "state")
		if state == 'disabled':
			self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
			self.fmtx_iface.Set("com.nokia.FMTx.Device", "state", dbus.String(u'%s' % 'enabled', variant_level=1))
		else:
			self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
			self.fmtx_iface.Set("com.nokia.FMTx.Device", "state", dbus.String(u'%s' % 'disabled', variant_level=1))

	def nextDown(self): #change button color
		self.ui.btnNext.setStyleSheet("background-color: " + self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/next_small.png);")

	def nextUp(self): #change button color
		self.ui.btnNext.setStyleSheet("background-color: rgb(0, 0, 0);\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/next_small.png);")

	def doPlayPause( self, signal=None ): #Play if stopped, pause if playing, receive signals from Backend if changed from somewhere else (ie: headphones)
		if self.player.playing:
			if signal != "play":
				self.player.stopPlayBack('pause')
				self.player.playing = False
				self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color +";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
		elif self.player.playlist != []:
			if signal != "pause":
				self.player.startPlayBack()
				self.player.playing = True
				self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color +";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
		else:
			self.playStation()
			self.player.playing = True
			self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color +";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")

	def doLove( self ): #Love this song
		self.player.loveHateRating = True
		self.player.loveHateSong = 1
		self.player.curSong[1]["rating"] = 1
		self.ui.btnLove.setStyleSheet("background-color: "+ self.c.color +";\n"
		"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbup_small.png);")

	def doHate( self ): #Hate this song
		self.player.loveHateRating = False
		self.player.loveHateSong = 1
		self.skip = 1

	def sigLoveHate( self ): #Signal for Love/Hate from backend
		if not self.player.respLH:
			self.ui.btnLove.setStyleSheet("background-color: "+ self.c.color +";\n"
			"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbup_small.png);")
		if self.skip:
			self.playNextSong()

	def hateDown(self): #change button color
		self.ui.btnHate.setStyleSheet("background-color: " + self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbdwn_small.png);")

	def hateUp(self): #change button color
		self.ui.btnHate.setStyleSheet("background-color: rgb(100, 100, 100);\nborder: rgb(255, 255, 255);\nbackground-image: url(:/img/img/thumbdwn_small.png);")

	def sigAuth( self ): #Signal for authentication success/failure.
		if self.player.success:
			self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authentication succeeded.  Wait for station list to appear in drop down below and select a station.</font></td><tr></table>" )
			self.count = 0
			self.player.getStatlist = 1
		else:
			self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authentication failed.  Please try again.</font></td><tr></table>" )
			self.c.user = ""
			self.c.password = ""
			self.auth()

	def auth( self ): #Start authentication process.
		if self.os == "RX-51":
			if not self.portrait:
				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
			else:
				self.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
		if self.count>2:
			self.closeEvent()
		self.count += 1
		ok = True #In case we got the user/pass from config
		if self.c.user == "":
			self.c.user,ok = QtGui.QInputDialog.getText(self,"Username", "Pandora Username:", QLineEdit.Normal)
		if not ok: #Username was cancelled.. they must be done.
			self.closeEvent()
		if self.c.password == "": 
			self.c.password,ok = QtGui.QInputDialog.getText(self,"Password", "Pandora Password:", QLineEdit.Password)
		if not ok: #Password was cancelled.. they must be done.
			self.closeEvent()
		self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authenticating to Pandora as "+self.c.user+" ... </font></td><tr></table>" )
		self.player.user = self.c.user
		self.player.password = self.c.password
		self.player.doAuth = 1
		self.authenticated = True
		self.c.write()
		if self.os == "RX-51":
			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)

	def playStation( self ): #Play currently selected station
		if len(self.createStations)>0: #If we're creating a station, finish creating and not playing selected station
			self.player.createStationId = self.createStations[self.ui.listStations.currentIndex()]['musicId']
			self.createStations = []
			return
		
		if self.player.playStation > 0:
			stationId = self.player.playStation
			self.player.playStation = 0
		else:
			stationId = self.player.stations[self.ui.listStations.currentIndex()]['stationId']	
		self.player.curStation = stationId
		self.player.playlist = []
		self.player.getMoreSongs()
		self.playNextSong()
		if stationId != self.c.lastplayed:
			f = open(os.path.expanduser("~")+'/.config/pyradio/lastplayed', 'w')
			f.write(self.player.curStation)
			f.close()

	def getStations( self ): #Get Station List	
		self.count = 0
		self.ui.listStations.clear()
		self.ui.btnPlay.setEnabled( True )
		self.ui.listStations.setEnabled( True )
		self.ui.btnLove.setEnabled( True )
		self.ui.btnHate.setEnabled( True )
		self.ui.btnSleep.setEnabled( True )
		self.ui.btnNext.setEnabled( True )
		self.ui.lblTime.setWordWrap( True )

		self.buildPlaylist()

		if self.player.playStation > 0:
			self.playStation()

	def buildPlaylist( self ): #Populate the station list box
		for s in self.player.stations: #Stations need to be scanned for QuickMix seperately
			if s["isQuickMix"]:
				qMixIds = s["quickMixStationIds"]
				break

		for s in self.player.stations: #Add stations
			#print s
			#print ""
			sName = s["stationName"]
			for qMixId in qMixIds:
				if qMixId == s["stationId"]:
					sName += " *" #If we're in the QuickMix list we add a *
			self.ui.listStations.addItem(sName)
			if self.player.playStation>0 and s["stationId"] == self.player.playStation:
				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)	
	
			if self.player.curStation>0 and s["stationId"] == self.player.curStation:
				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)	

	def setUserPass( self ): #Reset username/password and start authenticating
		self.c.user = ""
		self.c.password = ""
		self.auth()

	def playNextSong( self ): #Go to next song in playlist
		try:
			next = self.player.playlist.pop( 0 )
		except:
			self.player.getMoreSongs()
			next = self.player.playlist.pop( 0 )

		print "Playing Next Song"
		self.skip = 0
		self.player.curSong = next
		self.player.playSong ( next )
		self.setUi ( next )
		if self.player.playlist == []:
			#Out of songs, grab some more while playing
			self.player.getPlaylist = 1
	
	def setUi ( self, next=None ): #Update the album graphic and set Love graphic
		art = next[1]["art"]
		self.ui.albumArt.setHtml("<table width='100%' height='100%'><tr><td vertical-align='middle' style='text-align: center'><img style='width: 80%; height: 100%' src='"+art+"'></td></tr></table>")
		self.ui.lblSong.setText("\"" + next[1]["title"] + "\"\nArtist: " + next[1]["artist"])
		if next[1]["rating"] > 0:
			self.ui.btnLove.setStyleSheet("background-color: "+ self.c.color + ";\n"
			"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbup_small.png);")
		else:
			self.ui.btnLove.setStyleSheet("background-image: url(:/img/img/thumbup_small.png);\nborder: rgb(0, 0, 0);\nbackground-color: rgb(100, 100, 100);")

		self.ui.btnPlay.setStyleSheet("background-image: url(:/img/img/Play_small.png);\nborder: rgb(0, 0, 0);\nbackground-color: "+ self.c.color + ";")
		self.ui.btnNext.setEnabled( True )

	def stop( self ): #Stop Playing, not currently implemented
		self.player.playing = False

	def closeEvent(self, event=None): #Exit application gracefully.. somewhat.
		self.player.run = 0
		self.player.close()
		del self.ui
		del self.player
		quit()

if __name__ == '__main__':
	if not ( os.path.exists( os.path.dirname(sys.argv[0])+"/libpiano/crypt_key_input.h" ) \
			and os.path.exists( os.path.dirname(sys.argv[0])+"/libpiano/crypt_key_output.h" ) ):
		ok = QtGui.QMessageBox.warning(None, "Pandora", "Missing encription key files." )
	else:
		app = QtGui.QApplication(sys.argv)
		panda = Panda()
		panda.show()
		sys.exit(app.exec_())
